//
//  HUDLayer.mm
//  Raycast
//
//  Created by Ray Wenderlich on 8/18/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "HUDLayer.h"
#import "ActionLayer.h"

@implementation HUDLayer
@synthesize delegate;

- (id)init {
 	if ((self = [super init])) {
		self.isTouchEnabled = YES;
        
        leftButton = [CCSprite spriteWithFile:@"leftButton.png"];
        rightButton = [CCSprite spriteWithFile:@"rightButton.png"];
        jumpButton = [CCSprite spriteWithFile:@"jumpButton.png"];
        
        buttons = [[NSArray alloc ] initWithObjects:leftButton, rightButton, jumpButton, nil];
        
        for (CCSprite *s in buttons) {
            s.opacity = 127;
        }
        
        CGSize winSize = [CCDirector sharedDirector].winSize;
        
        _statusLabel = [CCLabelBMFont labelWithString:@"" fntFile:@"Arial.fnt"];
        leftButton.position = ccp(50, 50);
        rightButton.position = ccp(150, 50);
        jumpButton.position = ccp(440, 50);
        
        leftButton.scale = 0.5;
        rightButton.scale = 0.5;
        jumpButton.scale = 0.5;
        
        
        [self addChild:jumpButton];
        [self addChild:leftButton];
        [self addChild:rightButton];
        
        _statusLabel.position = ccp(winSize.width* 0.85, winSize.height * 0.9);
        [self addChild:_statusLabel]; 
        
        iCadeReaderView *icrv = [[iCadeReaderView alloc] initWithFrame:CGRectZero];
        EAGLView *root = [[CCDirector sharedDirector] openGLView];
        [root addSubview:icrv];
        icrv.active = YES;
        icrv.delegate = self;
        
    }
    return self;
}

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {    
    for (UITouch *t in touches) {
        
        CGPoint touchLocation = [self convertTouchToNodeSpace:t];
        
        for (CCSprite *s in buttons) {
            if (CGRectContainsPoint(s.boundingBox, touchLocation)) {
                s.opacity = 255;
                int buttIndex = [buttons indexOfObject:s];
                if (buttIndex == 2) {
                    [delegate heroJump];
                } else if (buttIndex == 1) {
                    [delegate heroMove:kDirectionRight];
                } else if (buttIndex == 0) {
                    [delegate heroMove:kDirectionLeft];
                }
                
            } 
        }
    }
}

- (void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    for (UITouch *t in touches) {
        
        CGPoint touchLocation = [self convertTouchToNodeSpace:t];
        
        //get previous touch and convert it to node space
        CGPoint previousTouchLocation = [t previousLocationInView:[t view]];
        CGSize screenSize = [[CCDirector sharedDirector] winSize];
        previousTouchLocation = ccp(previousTouchLocation.x, screenSize.height - previousTouchLocation.y);
        
        for (CCSprite *s in buttons) {
            if (CGRectContainsPoint(s.boundingBox, previousTouchLocation) && 
                !CGRectContainsPoint(s.boundingBox, touchLocation)) {
                
                s.opacity = 127;
                int buttIndex = [buttons indexOfObject:s];
                
                if (buttIndex == 1 || buttIndex == 0) {
                    [delegate heroMove:kDirectionNone];
                } 
            }
        }
        
        for (CCSprite *s in buttons) {
            if (!CGRectContainsPoint(s.boundingBox, previousTouchLocation) && 
                CGRectContainsPoint(s.boundingBox, touchLocation)) {
                
                s.opacity = 255;
                int buttIndex = [buttons indexOfObject:s];
                
                //We don't get another jump on a slide on, we want the player to let go of the button for another jump
                if (buttIndex == 1) {
                    [delegate heroMove:kDirectionRight];
                } else if (buttIndex == 0) {
                    [delegate heroMove:kDirectionLeft];
                }
                
            } 
        } 
    }
}

- (void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    
    for (UITouch *t in touches) {
        
        CGPoint touchLocation = [self convertTouchToNodeSpace:t];
        
        for (CCSprite *s in buttons) {
            if (CGRectContainsPoint(s.boundingBox, touchLocation)) {
                s.opacity = 127;
                int buttIndex = [buttons indexOfObject:s];
                
                if (buttIndex == 1 || buttIndex == 0) {
                    [delegate heroMove:kDirectionNone];
                } 
            }
        }
    }
}

- (void)setStatusString:(NSString *)string {
    _statusLabel.string = string;
}

- (void)restartTapped:(id)sender {
    
    // Reload the current scene
    CCScene *scene = [ActionLayer scene];
    [[CCDirector sharedDirector] replaceScene:[CCTransitionZoomFlipX transitionWithDuration:0.5 scene:scene]];
    
}

- (void)showRestartMenu:(BOOL)won {
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    
    NSString *message;
    if (won) {
        message = @"You win!";
    } else {
        message = @"You lose!";
    }
    
    CCLabelBMFont *label;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        label = [CCLabelBMFont labelWithString:message fntFile:@"Arial-hd.fnt"];
    } else {
        label = [CCLabelBMFont labelWithString:message fntFile:@"Arial.fnt"];
    }
    label.scale = 0.1;
    label.position = ccp(winSize.width/2, winSize.height * 0.6);
    [self addChild:label];
    
    CCLabelBMFont *restartLabel;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        restartLabel = [CCLabelBMFont labelWithString:@"Restart" fntFile:@"Arial-hd.fnt"];    
    } else {
        restartLabel = [CCLabelBMFont labelWithString:@"Restart" fntFile:@"Arial.fnt"];    
    }
    
    CCMenuItemLabel *restartItem = [CCMenuItemLabel itemWithLabel:restartLabel target:self selector:@selector(restartTapped:)];
    restartItem.scale = 0.1;
    restartItem.position = ccp(winSize.width/2, winSize.height * 0.4);
    
    CCMenu *menu = [CCMenu menuWithItems:restartItem, nil];
    menu.position = CGPointZero;
    [self addChild:menu z:10];
    
    [restartItem runAction:[CCScaleTo actionWithDuration:0.5 scale:1.0]];
    [label runAction:[CCScaleTo actionWithDuration:0.5 scale:1.0]];
    
}

-(void)buttonDown:(iCadeState)button {
    if (button == iCadeJoystickLeft) {
        [delegate heroMove:kDirectionLeft];
        leftButton.opacity = 255;
    } else if (button == iCadeJoystickRight) {
        [delegate heroMove:kDirectionRight];
        rightButton.opacity = 255;
    } else if (button == iCadeButtonA || 
               button == iCadeButtonB || 
               button == iCadeButtonC || 
               button == iCadeButtonD || 
               button == iCadeButtonE || 
               button == iCadeButtonF || 
               button == iCadeButtonG || 
               button == iCadeButtonH) {
        [delegate heroJump];
        jumpButton.opacity = 255;
    } 
}

-(void)buttonUp:(iCadeState)button {
    if (button == iCadeJoystickLeft) {
        [delegate heroMove:kDirectionNone];
        leftButton.opacity = 127;
    } else if (button == iCadeJoystickRight) {
        [delegate heroMove:kDirectionNone];
        rightButton.opacity = 127;
    } else if (button == iCadeButtonA || 
               button == iCadeButtonB || 
               button == iCadeButtonC || 
               button == iCadeButtonD || 
               button == iCadeButtonE || 
               button == iCadeButtonF || 
               button == iCadeButtonG || 
               button == iCadeButtonH) {
        jumpButton.opacity = 127;
    } 
}

@end
